package de.duehl.gameoflife.logic;

import de.duehl.basics.logic.ErrorHandler;
import de.duehl.basics.logic.Version;

import de.duehl.gameoflife.data.Options;
import de.duehl.gameoflife.logic.animationloop.AnimationLoop;
import de.duehl.gameoflife.logic.field.GameField;
import de.duehl.gameoflife.ui.GameOfLifeGui;
import de.duehl.gameoflife.ui.Gui;

/**
 * Diese Klasse stellt die Logik des Game of Life Spiels dar.
 *
 * @version 1.01     2015-11-10
 * @author Christian Dühl
 */

public class GameOfLifeLogic implements Logic {

    /** Programmversion. */
    private final Version version;

    /** Spielfeld. */
    private final GameField field;
    //private volatile GameField field;
    /*
     * Da es sowohl in GameOfLifePanel als auch in der AnimationLoop benötigt
     * wird, erstellen wir es hier. Da es in zwei Threads verwendet wird, ist es
     * volatile, auch wenn nur die Game-Loop Veränderungen daran vornimmt.
     */

    /** Verbindung zur grafischen Oberfläche. */
    private final Gui gui;

    /** Die Animation-Loop. */
    private final AnimationLoop animationLoop;

    /** Programmoptionen. */
    private final Options options;

    /** Sessionmanager für Daten, die zwischen Sessions erhalten bleiben. */
    private final SessionManager sessionManager;

    /**
     * Konstruktor.
     *
     * @param version
     *            Programmversion.
     */
    public GameOfLifeLogic(final Version version) {
        this.version = version;

        field = new GameField(50, 50);

        options = new Options(); // werden schon im Konstruktor der Gui benötigt.
        gui = new GameOfLifeGui(this);

        /* Optionen laden: */
        sessionManager = new SessionManager((ErrorHandler) gui, options);
        sessionManager.load();

        gui.show();
        animationLoop = new AnimationLoop(field);
    }

    /** Getter für die Programmversion. */
    @Override
    public Version getVersion() {
        return version;
    }

    /** Getter für das Spielfeld. */
    @Override
    public GameField getField() {
        return field;
    }

    /** Beendet das Programm. */
    @Override
    public void quit() {
        boolean quit = gui.askUserToQuit();
        if (quit) {
            internalQuit();
        }
    }

    /** Beendet das Programm. */
    private void internalQuit() {
        if (animationLoop != null) {
            animationLoop.stop();
        }
        sessionManager.save();

        gui.quit();
    }

    /** Getter für die Animation-Loop. */
    @Override
    public AnimationLoop getAnimationLoop() {
        return animationLoop;
    }

    /** Leert das Spielfeld. */
    @Override
    public void clearGameField() {
        field.clear();
    }

    /** Getter für die Optionen. */
    @Override
    public Options getOptions() {
        return options;
    }

    /** Zeigt den Optionendialog an. */
    @Override
    public void showOptionDialog() {
        gui.showOptionDialog(options);
    }

}
