package de.duehl.gameoflife.logic;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Properties;

import de.duehl.gameoflife.data.Options;

import de.duehl.basics.io.FileHelper;
import de.duehl.basics.logic.ErrorHandler;

/**
 * Dieses Klasse regelt den Umgang mit Informationen, die zwischen zwei Läufen
 * des Programms bewahrt werden sollen. Sie stellt Methoden bereit, diese Daten
 * zu speichern und zu lesen.
 *
 * @version 1.01     2013-11-05
 * @author Christian Dühl
 */

public class SessionManager {

    /** Ein Options-Objekt, dort werden die Optionen des Programms verwaltet. */
    private final Options options;

    /** Objekt, das die Fehlerbehandlung durchführt. */
    private final ErrorHandler error;

    /**
     * Name der Datei, in der die Informationen gespeichert und aus der gelesen
     * wird.
     */
    protected final String sessionFilename;

    /**
     * Konstruktor.
     *
     * @param error
     *            Objekt, das die Fehlerbehandlung durchführt.
     * @param options
     *            Einstellungsobjekt, dessen Daten gesichert / gelesen werden
     *            sollen.
     */
    public SessionManager(ErrorHandler error, Options options) {
        this.error = error;
        this.options = options;

        String path = FileHelper.getAndPerhapsCreateDirectoryAtHome(
                ".GameOfLife");
        sessionFilename = path + File.separator + "gameOfLifeSession.xml";
    }

    /** Lädt die Optionen. */
    public void load() {
        Properties properties = readProperties();
        if (null != properties) {
            loadOptions(properties);
        }
    }

    /**
     * Lädt die Optionen und Gui-Einstellungen und setzt die Werte bzw.
     * Eigenschaften in den Gui-Elementen. Hier werden die eigentlichen Daten
     * aus den Properties gelesen.
     */
    private void loadOptions(Properties properties) {
        if (properties.containsKey("options.livingCellColor")) {
            String livingCellColor =
                    properties.getProperty("options.livingCellColor");
            options.setLivingCellColor(livingCellColor);
        }
        if (properties.containsKey("options.deadCellColor")) {
            String deadCellColor =
                    properties.getProperty("options.deadCellColor");
            options.setDeadCellColor(deadCellColor);
        }
    }

    /**
     * Liest das Sessionfile ein und gibt ein passendes Properties Objekt zurück
     * oder null, falls es nicht gefunden wurde oder es beim Einlesen Probleme
     * gab.
     *
     * @return Properties des Sessionfiles oder null.
     */
    private Properties readProperties() {
        Properties properties = new Properties();

        /* Properties einlesen: */
        try {
            properties.loadFromXML(new FileInputStream(sessionFilename));
        }
        catch (FileNotFoundException e) {
            /*
             * Hier keine Fehlermeldung, da es ja nicht schlimm ist, wenn es die
             * Datei noch nicht gibt!
             */
            properties = null;
        }
        catch (IOException e) {
            error.error("Sessionfile IOException beim Lesen: " + e.getMessage());
            properties = null;
        }

        return properties;
    }

    /** Speichert die Optionen. */
    public void save() {
        Properties properties = new Properties();
        saveOptions(properties);
        writeProperties(properties);
    }

    /**
     * Speichert die Optionen und Gui-Einstellungen. Hier werden die
     * eigentlichen Daten in die Properties geschrieben.
     */
    private void saveOptions(Properties properties) {
        properties.setProperty("options.livingCellColor",
                options.getLivingCellColor());
        properties.setProperty("options.deadCellColor",
                options.getDeadCellColor());
    }

    /** Properties als XML-Datei abspeichern. */
    private void writeProperties(Properties properties) {
        try {
            properties.storeToXML(new FileOutputStream(sessionFilename), "");
        }
        catch (FileNotFoundException e) {
            error.error("Sessionfile " + sessionFilename
                    + " wurde  nicht gefunden: " + e.getMessage());
        }
        catch (IOException e) {
            error.error("Sessionfile " + sessionFilename
                    + " konnte nicht geschrieben werden: " + e.getMessage());
        }
    }

}
