package de.duehl.gameoflife.logic.animationloop;

import de.duehl.gameoflife.logic.field.GameField;

public class AnimationLoop implements Runnable {

    /**
     * Das Objekt, das über neu berechnete Generationen informiert werden
     * möchte.
     */
    private AnimationStepWatcher watcher;

    /** Ablaufgeschwindigkeit. Je kleiner, desto schneller. */
    private volatile int speed;

    /**
     * Ist wahr, solange die Game-Loop läuft, also solange das Spiel nicht
     * beendet ist.
     */
    private volatile boolean running; /* wird von außen in quit geändert! */

    /** Spielfeld. */
    private volatile GameField field;
    /*
     * Da es sowohl in GameOfLifePanel als auch in der AnimationLoop benötigt
     * wird, erstellen wir es hier. Da es in zwei Threads verwendet wird, ist es
     * volatile, auch wenn nur die Game-Loop Veränderungen daran vornimmt.
     */

    /**
     * Konstruktor.
     *
     * @param field
     *            Spielfeld.
     */
    public AnimationLoop(GameField field) {
        this.field = field;
    }

    /**
     * Setzt den Beobachter.
     *
     * @param watcher
     */
    public void setAnimationStepWatcher(AnimationStepWatcher watcher) {
        this.watcher = watcher;
    }

    /** Stellt die Geschwindigkeit der Animation ein. */
    public void setSpeed(int speed) {
        this.speed = speed;
    }

    /** Startet die Animation - Aufruf aus dem EDT */
    public void start() {
        running = true;
        /*
         * Da running volatile ist, macht es nichts, diese Variable im EDT zu
         * verändern.
         */
        Thread thread = new Thread(this);
        thread.start();
    }

    /** Hält die Animation an. - Aufruf aus dem EDT */
    public void stop() {
        running = false;
        /*
         * Da running volatile ist, macht es nichts, diese Variable im EDT zu
         * verändern.
         */
    }

    /**
     * Hier befindet sich die eigentliche Animation-Loop. Sie wird im eigenen
     * Thread - dem Animation-Loop-Thread - gestartet.
     */
    @Override
    public void run() {
        long lastTime = System.currentTimeMillis();
        while (running) {
            long actualTime = System.currentTimeMillis();
            long diff = actualTime - lastTime;
            if (diff >= speed) {
                lastTime = actualTime;
                animationLoopStep();
            }
            sleep();
        }
    }

    /** Schläft 10 Millisekunden. */
    private void sleep() {
        try {
            Thread.sleep(10);
        }
        catch (InterruptedException e) {
            e.printStackTrace();
            // Sollte nicht auftreten, da der Thread nicht aufgehoben wird, so
            // kann er auch nicht unterbrochen werden.
        }
    }

    /** Führt einen Schritt der Animation aus. */
    private void animationLoopStep() {
        field.calculateNextGeneration();
        if (watcher != null) {
            watcher.nextGenerationCalculated();
        }
    }

}
