package de.duehl.gameoflife.logic.field;

import de.duehl.gameoflife.logic.randomBoolean.BooleanInitializer;
import de.duehl.gameoflife.logic.randomBoolean.RandomBooleanInitializer;

/**
 * Diese Klasse stellt das Spielfeld vom Spiel des Lebens dar.
 *
 * @version 1.01     2013-10-31
 * @author Christian Dühl
 */

public class GameField {

    /** Spielfeld */
    private final boolean[][] field;

    /** Breite des Spielfeldes. */
    private final int width;

    /** Höhe des Spielfeldes. */
    private final int height;

    /** Objekt, dass Booleanwerte initialisiert. */
    private BooleanInitializer booleanInitializer;

    /**
     * Konstruktor
     *
     * @param width
     *            Breite des Spielfeldes.
     * @param height
     *            Höhe des Spielfeldes.
     */
    public GameField(int width, int height) {
        field = new boolean[width][height];
        this.width = width;
        this.height = height;
        booleanInitializer = new RandomBooleanInitializer();
    }

    /**
     * Konstruktor
     *
     * @param initialField
     *            Vorbelegtes, regelmäßiges Spielfeld.
     */
    public GameField(boolean[][] initialField) {
        checkBooleanField(initialField);
        field = initialField;
        height = field.length;
        width = field[0].length;

        booleanInitializer = new RandomBooleanInitializer();
    }

    /**
     * Überprüft ein Feld aus Booleans. es darf nicht leer sein und die Zeilen
     * müssen alle gleich lang sein.
     *
     * @param initialField
     *            Zu überprüfendes Feld.
     * @throws IllegalArgumentException
     *             wenn das Feld leer ist oder es Zeilen mit unterschiedlicher
     *             Länge gibt.
     */
    private void checkBooleanField(boolean[][] initialField) {
        int height = initialField.length;
        int width = 0;
        if (height > 0) {
            width = initialField[0].length;
            for (int y=1; y<height; ++y) {
                if (width != initialField[y].length) {
                    throw new IllegalArgumentException(
                            "Feldbreiten unregelmäßig");
                }
            }
        }
        else {
            throw new IllegalArgumentException("Höhe darf nicht null sein");
        }
        if (width == 0) {
            throw new IllegalArgumentException("Breite darf nicht null sein");
        }
    }

    /** Leert das Spielfeld (alle Zellen sind tot). */
    public void clear() {
        for (int y=0; y<height; y++) {
            for (int x=0; x<width; x++) {
                field[x][y] = false;
            }
        }
    }

    /** Getter für die Breite des Spielfeldes. */
    public int getWidth() {
        return width;
    }

    /** Getter für die Höhe des Spielfeldes. */
    public int getHeight() {
        return height;
    }

    /**
     * Gibt an, ob das angegebene Spielfeld gefüllt ist.
     *
     * @param x
     *            x-Koordinate
     * @param y
     *            y-Koordinate
     * @return true genau dann, wenn das Spielfeld an der angegebenen Koordinate
     *         lebt.
     */
    public boolean isCellAlive(int x, int y) {
        checkCoordinatesAndThrow(x, y);
        return field[x][y];
    }

    /**
     * Überprüft übergebene Koordinaten und wirft im Fehlerfall eine
     * IllegalArgumentException.
     *
     * @param x
     *            x-Koordinate
     * @param y
     *            y-Koordinate
     * @throws IllegalArgumentException
     *             wenn die Koordinaten außerhalb es Feldes liegen.
     */
    private void checkCoordinatesAndThrow(int x, int y) {
        if (x < 0) {
            throw new IllegalArgumentException("x-Wert zu klein");
        }
        if (x >= width) {
            throw new IllegalArgumentException("x-Wert zu groß");
        }
        if (y < 0) {
            throw new IllegalArgumentException("y-Wert zu klein");
        }
        if (y >= height) {
            throw new IllegalArgumentException("y-Wert zu groß");
        }
    }

    /**
     * Überprüft übergebene Koordinaten und wirft im Fehlerfall eine
     * IllegalArgumentException.
     *
     * @param x
     *            x-Koordinate
     * @param y
     *            y-Koordinate
     * @return true, wenn die Koordinaten zulässig sind.
     */
    private boolean checkCoordinates(int x, int y) {
        if (x < 0) {
            return false;
        }
        if (x >= width) {
            return false;
        }
        if (y < 0) {
            return false;
        }
        if (y >= height) {
            return false;
        }

        return true;
    }

    /**
     * Setzt den Wert einer Zelle.
     *
     * @param x
     *            x-Koordinate
     * @param y
     *            y-Koordinate
     * @param living
     *            Gibt an, ob die Zelle lebt.
     */
    void setCell(int x, int y, boolean living) {
        checkCoordinatesAndThrow(x, y);
        field[x][y] = living;
    }

    /**
     * Für Testzwecke können andere als zufällige Initialisierungen übergeben
     * werden.
     *
     * @param booleanInitializer Boolean-Initialisierer.
     */
    void setBooleanInitializer(BooleanInitializer booleanInitializer) {
        this.booleanInitializer = booleanInitializer;
    }

    /**
     * Erzeugt eine zufällige Feldbelegung, zumindest falls nicht ein nicht
     * zufälliger BooleanInitializer gesetzt wurde.
     */
    public void createRandomField() {
        for (int y=0; y<height; y++) {
            for (int x=0; x<width; x++) {
                field[x][y] = booleanInitializer.getInitialValue();
            }
        }
    }

    /** Berechnet die nächste Generation: */
    public void calculateNextGeneration() {
        boolean[][] nextField = new boolean[width][height];
        for (int y=0; y<height; y++) {
            for (int x=0; x<width; x++) {
                int livingNeighbourCount = countLivingNeighbours(x, y);
                /* Betrachtung für lebende Zellen: */
                if (field[x][y]) {
                    if (2 == livingNeighbourCount || 3 == livingNeighbourCount) {
                        nextField[x][y] = true;
                    }
                    else { // < 2 oder > 3
                        nextField[x][y] = false;
                    }
                }
                /* Betrachtung für tote Zellen: */
                else {
                    if (3 == livingNeighbourCount) {
                        nextField[x][y] = true;
                    }
                    else {
                        nextField[x][y] = false;
                    }
                }
            }
        }
        for (int y=0; y<height; y++) {
            for (int x=0; x<width; x++) {
                field[x][y] = nextField[x][y];
            }
        }
    }

    /**
     * Zählt die lebendigen Nachbarn der Zelle
     *
     * @param x
     *            x-Koordinate
     * @param y
     *            y-Koordinate
     * @return Anzahl der lebenden Nachbarn.
     */
    int countLivingNeighbours(int x, int y) {
        int livingNeighbourCount = 0;
        for (int deltaY = -1; deltaY < 2; ++ deltaY) {
            for (int deltaX = -1; deltaX < 2; ++ deltaX) {
                int testX = x + deltaX;
                int testY = y + deltaY;
                if (testX == x && testY == y) {
                    continue; // Die Zelle selbst ist nicht ihr Nachbar
                }
                if (checkCoordinates(testX, testY)) {
                    if (field[testX][testY]) { // nicht isLivingCell, weil schon überprüft wurde!
                        ++livingNeighbourCount;
                    }
                }
            }
        }
        return livingNeighbourCount;
    }

    /**
     * Fügt ein Teilfeld in das Spielfeld an der gegebenen Position ein.
     *
     * @param subField
     *            Teilfeld, das eingefügt werden soll.
     * @param atX
     *            x-Koordinate
     * @param atY
     *            y-Koordinate
     */
    public void insertSubField(boolean[][] subField, int atX, int atY) {
        /* Ist das übergebene Feld nicht leer und gleichmäßig? */
        checkBooleanField(subField);

        /* Startpunkt (= obere linke Ecke)innerhalb des Feldes? */
        checkCoordinatesAndThrow(atX, atY);

        int subHeight = subField.length;
        int subWidth = subField[0].length;

        /* Prüfung, ob das Teilfeld ganz in das Feld passt: */
        int lowerRightX = atX + subWidth-1;
        int lowerRightY = atY + subHeight-1;
        checkCoordinatesAndThrow(lowerRightX, lowerRightY);

        /* Inhalt in das Feld kopieren: */
        for (int y=0; y<subHeight; y++) {
            for (int x=0; x<subWidth; x++) {
                field[atX + x][atY + y] = subField[x][y];
            }
        }
    }

    /** Darstellung als String. Hauptsächlich zum Debuggen. */
    @Override
    public String toString() {
        StringBuilder builder = new StringBuilder();
        builder.append("GameField " + width + " x " + height + "\n");
        for (int y=0; y<height; y++) {
            builder.append(String.format("%4d ", y));
            for (int x=0; x<width; x++) {
                builder.append(String.format("%s ", (field[x][y] ? "#" : "_")));
            }
            builder.append("\n");
        }
        builder.append("    ");
        for (int x=0; x<width; x++) {
            builder.append(String.format("%d ", x));
        }
        builder.append("\n");

        return builder.toString();
    }

    /**
     * Ändert den Zustand einer Zelle.
     *
     * @param x
     *            x-Koordinate
     * @param y
     *            y-Koordinate
     */
    public void toggle(int x, int y) {
        checkCoordinatesAndThrow(x, y);
        field[x][y] = !field[x][y];
    }

    /**
     * Setzt den Zustand der gegebenen Zelle auf lebend oder tot.
     *
     * @param x
     *            x-Koordinate
     * @param y
     *            y-Koordinate
     * @param living
     *            Zustand der Zelle.
     */
    public void setCellLiving(int x, int y, boolean living) {
        checkCoordinatesAndThrow(x, y);
        field[x][y] = living;
    }

}
