package de.duehl.gameoflife.ui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.EventQueue;
import java.awt.Image;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.net.URL;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JColorChooser;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.SwingConstants;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import de.duehl.swing.ui.colors.Colorizer;
import de.duehl.swing.ui.dialogs.ErrorDialog;
import de.duehl.swing.ui.dialogs.html.ShowHTMLFrame;
import de.duehl.swing.ui.layout.ExtendedFlowLayout;
import de.duehl.swing.ui.layout.VerticalLayout;
import de.duehl.swing.ui.listener.ClosingWindowListener;
import de.duehl.swing.ui.resources.IconLoader;
import de.duehl.basics.logic.ErrorHandler;
import de.duehl.basics.logic.Version;
import de.duehl.basics.text.Text;
import de.duehl.gameoflife.data.Options;
import de.duehl.gameoflife.logic.Logic;
import de.duehl.gameoflife.logic.animationloop.AnimationLoop;
import de.duehl.gameoflife.logic.animationloop.AnimationStepWatcher;
import de.duehl.gameoflife.ui.dialogs.OptionDialog;
import de.duehl.gameoflife.ui.dialogs.QuitDialog;
import de.duehl.gameoflife.ui.panel.GameOfLifePanel;
import de.duehl.gameoflife.ui.resources.IconDefinitions;

/**
 * Diese Klasse erzeugt die grafische Oberfläche von Game of Life.
 *
 * @version 1.01     2019-07-08
 * @author Christian Dühl
 */

public class GameOfLifeGui implements Gui, ErrorHandler {

    /** Zugriff auf die Programmlogik */
    private final Logic logic;

    /** Farbverwaltung für die Gui. */
    private final Colorizer colorizer;

    /** Mainframe. */
    private final JFrame frame;

    /** Bild des Programms. */
    private final Image programImage;

    /** Panel auf dem das Spielfeld dargestellt wird. */
    private final GameOfLifePanel gameOfLifePanel;

    /** Gib an, ob der Edit-Mode aktiv ist. */
    private boolean editMode;

    /** Gib an, ob der Run-Mode aktiv ist. */
    private boolean runMode;

    /* Gui Elemente */
    private final JButton editButton;
    private final JButton nextGenerationButton;
    private final JButton runButton;
    private final JSlider slider;

    /**
     * Konstruktor.
     *
     * @param logic
     *            Verbindung zur Logik.
     */
    public GameOfLifeGui(Logic logic) {
        this.logic = logic;
        colorizer = new GameOfLifeColorizer();

        programImage = loadProgrammImage();

        frame = new JFrame();
        gameOfLifePanel = new GameOfLifePanel(logic.getField(), logic.getOptions());
        nextGenerationButton = new JButton();
        editButton = new JButton();
        slider = new JSlider();
        runButton = new JButton();

        EventQueue.invokeLater(new Runnable() {
            @Override
            public void run() {
                createGui();
                AnimationLoop animationLoop = logic.getAnimationLoop();
                animationLoop.setAnimationStepWatcher(
                        (AnimationStepWatcher) gameOfLifePanel);
                new GameOfLifeMenu(colorizer, logic, frame, GameOfLifeGui.this);
            }
        });
    }

    /** Lädt das Bild des Programms. */
    private Image loadProgrammImage() {
        IconDefinitions iconDefinitions = new IconDefinitions();
        IconLoader iconLoader = iconDefinitions.createIconLoader();
        return iconLoader.loadProgramIconImage();
    }

    /** Erstellt die grafische Oberfläche. */
    private void createGui() {
        colorizer.setColors(frame);
        frame.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
        frame.setIconImage(programImage);
        frame.setLayout(new BorderLayout());
        //frame.setResizable(false);
        frame.addWindowListener(new ClosingWindowListener(logic));
        String title = "Game of Life "
                + logic.getVersion().getVersion() + " vom "
                + logic.getVersion().getDate();
        frame.setTitle(title);

        Dimension dimension = new Dimension(800, 840);
        frame.setPreferredSize(dimension);
        frame.setLocation(300, 200);


        /* Linken Teil anlegen: */
        frame.add(createUperPart(), BorderLayout.NORTH);

        /* Mittleren Teil anlegen: */
        frame.add(createCenterPart(), BorderLayout.CENTER);

        //GameLoopGui gameLoopGui = new GameLoopGui(frame, programImage,
        //        pentrisPanel, nextStonePanel, pointsLabel, highscorePanel,
        //        colorizer);
        //logic.setGameLoopGui(gameLoopGui);
    }

    /** Erstellt den oberen Bereich mit den Schaltern. */
    private Component createUperPart() {
        JPanel panel = new JPanel();
        panel.setLayout(new ExtendedFlowLayout(10));
        colorizer.setColors(panel);
        //colorizer.setStandardBorder(panel, "upper");

        addNextGenerationButton(panel);
        addEditButton(panel);
        addSlider(panel);
        addRunButton(panel);
        addColorButtons(panel);

        return panel;
    }

    /**
     * Fügt den Schalter für die nächste Generation des Spielfelds zum
     * übergebenen Panel.
     *
     * @param panel
     *            Panel, in den der Schalter eingefügt wird.
     */
    private void addNextGenerationButton(JPanel panel) {
        nextGenerationButton.setText(
                "<html><center><small>compute next<br>generation</small></center></html>");
        colorizer.setColors(nextGenerationButton);
        nextGenerationButton.setFocusable(false);
        nextGenerationButton.setVerticalAlignment(SwingConstants.CENTER);
        nextGenerationButton.setHorizontalAlignment(SwingConstants.CENTER);
        panel.add(nextGenerationButton);

        nextGenerationButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                nextGeneration();
            }
        });
    }

    /**
     * Fügt den Schalter zum Bearbeiten des Spielfelds zum übergebenen Panel.
     *
     * @param panel
     *            Panel, in den der Schalter eingefügt wird.
     */
    private void addEditButton(JPanel panel) {
        editButton.setText("<html><center>edit<br>game field</center></html>");
        colorizer.setColors(editButton);
        editButton.setFocusable(false);
        editButton.setToolTipText(
                "Wenn aktiv, Feld per Rechtsklick auf diesen Schalter löschen.");
        panel.add(editButton);

        editButton.addMouseListener(new MouseListener() {
            @Override
            public void mouseReleased(MouseEvent event) {
                if (MouseEvent.BUTTON1 == event.getButton()) {
                    switchEditMode();
                }
                else if (MouseEvent.BUTTON3 == event.getButton()) {
                    clearField();
                }
            }
            @Override
            public void mousePressed(MouseEvent e) {
            }
            @Override
            public void mouseExited(MouseEvent e) {
            }
            @Override
            public void mouseEntered(MouseEvent e) {
            }
            @Override
            public void mouseClicked(MouseEvent e) {
            }
        });
    }

    /**
     * Fügt den Slider für die Geschwindigkeit zum übergebenen Panel.
     *
     * @param panel
     *            Panel, in den der Schalter eingefügt wird.
     */
    private void addSlider(JPanel panel) {
        slider.setMinimum(0);
        slider.setMaximum(2000);
        slider.setValue(1000);
        AnimationLoop animationLoop = logic.getAnimationLoop();
        animationLoop.setSpeed(1000);

        colorizer.setColors(slider);
        slider.setToolTipText("Geschwindigkeit");
        slider.setPaintTicks(true);
        slider.setMajorTickSpacing(1000);
        slider.setMinorTickSpacing(100);
        slider.setPaintTrack(false);
        slider.addChangeListener(new ChangeListener() {
            @Override
            public void stateChanged(ChangeEvent event) {
                /*
                 * Kurze Überlegung zum Umdrehen des Wertes:
                 *     min             max
                 * x   0               100
                 * y   24              73
                 * z   -300            20
                 *
                 * r = min + (max - min) - (a - min) = max + min - a
                 *
                 * Fall x:
                 *    a = 0 => r = 100 + 0 - 0 = 100 richtig
                 *    a = 100 => r = 100 + 0 - 100 = 0 richtig
                 *    2 = 80 => r = 100 + 0 - 80 = 20 richtig
                 * Fall y:
                 *    a = 24 => r = 73 + 24 - 24 = 73 richtig
                 *    a = 73 => r = 73 + 24 - 73 = 24 richtig
                 *    a = 40 => r = 73 + 24 - 40 = 97 - 40 = 57 richtig
                 * Fall z:
                 *    a = -300 => r = 20 + (-300) - (-300) = 20 richtig
                 *    a = 20 => r = 20 + (-300) - 20 = -300 richtig
                 *    a = -10 => r = 20 + (-300) - (-10) = -300 + 30 = 270 richtig.
                 */
                int speed = slider.getValue();
                int min = slider.getMinimum();
                int max = slider.getMaximum();
                int revertedSpeed = max + min - speed;
                AnimationLoop animationLoop = logic.getAnimationLoop();
                animationLoop.setSpeed(revertedSpeed);
            }
        });

        panel.add(slider);
    }

    /**
     * Fügt den Schalter zum Laufen lassen der Animation zum übergebenen Panel.
     *
     * @param panel
     *            Panel, in den der Schalter eingefügt wird.
     */
    private void addRunButton(JPanel panel) {
        runButton.setText("<html><center>run<br>animation</center></html>");
        colorizer.setColors(runButton);
        runButton.setFocusable(false);
        panel.add(runButton);

        runButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                switchRunMode();
            }
        });
    }

    /**
     * Fügt die Schalter für die Farben der lebendigen und toten Zellen zum
     * übergebenen Panel.
     *
     * @param panel
     *            Panel, in den der Schalter eingefügt wird.
     */
    private void addColorButtons(JPanel panel) {
        JPanel subPanel = new JPanel();
        subPanel.setLayout(new VerticalLayout(2, VerticalLayout.BOTH));
        colorizer.setColors(subPanel);

        JButton livingColorButton = new JButton("color of living cells");
        colorizer.setColors(livingColorButton);
        livingColorButton.setFocusable(false);
        subPanel.add(livingColorButton);

        livingColorButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                changeLivingCellColor();
            }
        });

        JButton deadColorButton = new JButton("color of dead cells");
        colorizer.setColors(deadColorButton);
        deadColorButton.setFocusable(false);
        subPanel.add(deadColorButton);

        deadColorButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                changeDeadCellColor();
            }
        });

        panel.add(subPanel);
    }

    /** Verändert die Farbe der lebendigen Zellen. */
    private void changeLivingCellColor() {
        Options options = logic.getOptions();
        String hexColor = options.getLivingCellColor();
        Color oldColor = colorizer.hexColorToJavaColor(hexColor);
        Color selectedColor = JColorChooser.showDialog(
                frame,
                "Eine neue Farbe für lebende Zellen wählen:",
                oldColor
                );
        if (null != selectedColor) {
            String livingCellColor = colorizer.javaColorToHexColor(selectedColor);
            options.setLivingCellColor(livingCellColor);
            gameOfLifePanel.repaint();
        }
    }

    /** Verändert die Farbe der toten Zellen. */
    private void changeDeadCellColor() {
        Options options = logic.getOptions();
        String hexColor = options.getDeadCellColor();
        Color oldColor = colorizer.hexColorToJavaColor(hexColor);
        Color selectedColor = JColorChooser.showDialog(
                frame,
                "Eine neue Farbe für tote Zellen wählen:",
                oldColor
                );
        if (null != selectedColor) {
            String deadCellColor = colorizer.javaColorToHexColor(selectedColor);
            options.setDeadCellColor(deadCellColor);
            gameOfLifePanel.repaint();
        }
    }

    /** Berechnet die nächste Generation und zeigt sie an. */
    private void nextGeneration() {
        gameOfLifePanel.nextGeneration();
    }

    /** Schaltet den Edit-Mode an oder aus. */
    private void switchEditMode() {
        editMode = !editMode;
        gameOfLifePanel.setEditMode(editMode);
        if (editMode) {
            editButton.setForeground(Color.RED);
            nextGenerationButton.setEnabled(false);
            runButton.setEnabled(false);
            slider.setEnabled(false);
        }
        else {
            colorizer.setColors(editButton);
            nextGenerationButton.setEnabled(true);
            runButton.setEnabled(true);
            slider.setEnabled(true);
        }
    }

    /** Leert das Spielfeld. */
    private void clearField() {
        if (editMode) {
            logic.clearGameField();
            gameOfLifePanel.repaint();
        }
    }

    /** Schaltet den Run-Mode an oder aus. */
    private void switchRunMode() {
        runMode = !runMode;
        //gameOfLifePanel.setEditMode(editMode);
        AnimationLoop animationLoop = logic.getAnimationLoop();
        if (runMode) {
            runButton.setForeground(Color.RED);
            nextGenerationButton.setEnabled(false);
            editButton.setEnabled(false);
            //slider.setEnabled(true);
            animationLoop.start();
        }
        else {
            colorizer.setColors(runButton);
            nextGenerationButton.setEnabled(true);
            editButton.setEnabled(true);
            animationLoop.stop();
        }
    }

    /** Erstellt den mittleren Bereich mit dem Spielfeld. */
    private Component createCenterPart() {
        JPanel panel = new JPanel();
        panel.setLayout(new BorderLayout());
        colorizer.setColors(panel);
        //colorizer.setStandardBorder(panel, "center");

        colorizer.setColors(gameOfLifePanel);
        //gameOfLifePanel.repaint();
        panel.add(gameOfLifePanel, BorderLayout.CENTER);

        return panel;
    }
    /** Zeigt die Oberfläche an. */
    @Override
    public void show() {
        EventQueue.invokeLater(new Runnable() {
            @Override
            public void run() {
                frame.pack();
                frame.setVisible(true);
            }
        });
    }

    /** Fragt den Benutzer, ob das Programm beendet werden soll. */
    @Override
    public boolean askUserToQuit() {
        QuitDialog dialog = new QuitDialog(colorizer, frame, programImage);
        dialog.setVisible(true);
        boolean quit = dialog.doQuit();
        return quit;
    }

    /** Beendet die Oberfläche. */
    @Override
    public void quit() {
        frame.setVisible(false);
        frame.dispose();
    }

    /** Zeigt den Optionendialog an. */
    @Override
    public void showOptionDialog(Options options) {
        OptionDialog dialog = new OptionDialog(options, frame, colorizer,
                programImage);
        dialog.setVisible(true);
        gameOfLifePanel.repaint();
    }

    /** Zeigt das Über... Fenster aus dem Hilfemenü an */
    @Override
    public void about() {
        Version theVersion = logic.getVersion();
        String version = theVersion.getVersion();
        String date = theVersion.getDate();

        StringBuilder builder = new StringBuilder();
        builder.append("Game of Life nach Conway\n\n");
        builder.append("Autor: Christian Dühl\n");
        builder.append("Version: " + version + " vom " + date + "\n\n");
        String message = builder.toString();
        ImageIcon imageIcon = new ImageIcon(programImage);

        JOptionPane.showMessageDialog(frame, message,
                "Über Game Of Life",
                JOptionPane.INFORMATION_MESSAGE,
                imageIcon);
    }

    /** Zeigt die Hilfe aus dem Hilfemenü an */
    @Override
    public void help() {
        String doc = "de/duehl/gameoflife/documents/help/hilfe.html";
        URL url = this.getClass().getClassLoader().getResource(doc);
        ShowHTMLFrame dialog = new ShowHTMLFrame("Hilfe", programImage, frame.getLocation());
        dialog.showHtml(url);
        dialog.setVisible(true);
    }

    /** Zeigt die Liste der Programmänderungen aus dem Hilfemenü an */
    @Override
    public void changes() {
        String doc = "de/duehl/gameoflife/documents/changes.html";
        URL url = this.getClass().getClassLoader().getResource(doc);
        ShowHTMLFrame dialog = new ShowHTMLFrame("Änderungen", programImage, frame.getLocation());
        dialog.showHtml(url);
        dialog.setVisible(true);
    }

    /** Zeigt die Todo-Liste aus dem Hilfemenü an */
    @Override
    public void todo() {
        String doc = "de/duehl/gameoflife/documents/todo.html";
        URL url = this.getClass().getClassLoader().getResource(doc);
        ShowHTMLFrame dialog = new ShowHTMLFrame("Weiterentwicklung - Todos", programImage,
                frame.getLocation());
        dialog.showHtml(url);
        dialog.setVisible(true);
    }

    /** Gibt eine Warnung aus. */
    @Override
    public void warning(String message) {
        EventQueue.invokeLater(new Runnable() {
            @Override
            public void run() {
                /* Text umbrechen, dass er nicht zu lang wird: */
                String brokenMessage = Text.addLineBreaks(message, 80);
                JOptionPane.showMessageDialog(frame, brokenMessage, "Warnung",
                        JOptionPane.WARNING_MESSAGE);
            }
        });
    }

    /** Zeigt dem Benutzer einen Fehler an. */
    @Override
    public void error(String message) {
        error(message, null);
    }

    /** Zeigt dem Benutzer einen Fehler mit Exception an. */
    @Override
    public void error(String message, Exception exception) {
        //Image programImage = null;
        ErrorDialog dialog = new ErrorDialog(message, exception, programImage,
                null, colorizer.getForegroundColor(),
                colorizer.getBackgroundColor(), frame.getLocation());
        EventQueue.invokeLater(new Runnable() {
            @Override
            public void run() {
                dialog.setVisible(true);
            }
        });
    }

}
