package de.duehl.gameoflife.ui;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;

import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.KeyStroke;

import de.duehl.gameoflife.logic.Logic;

import de.duehl.swing.ui.colors.Colorizer;

/**
 * Diese Klasse erzeugt die Menü-Elemente der grafische Oberfläche von Game of
 * Life.
 *
 * @version 1.01     2018-04-05
 * @author Christian Dühl
 */

public class GameOfLifeMenu {

    /** Zugriff auf die Programmlogik */
    private final Logic logic;

    /** Farbverwaltung für die Gui. */
    private final Colorizer colorizer;

    /** Verbindung zur Gui. */
    private final Gui gui;

    /**
     * Konstruktor
     *
     * @param colorizer
     *            Farbverwaltung für die Gui.
     * @param logic
     *            Zugriff auf die Programmlogik des Servants
     * @param mainframe
     *            JFrame, zu dem das Menü eingetragen werden soll.
     * @param phase
     *            Phase in der das Programm ist.
     * @param gui
     *            Verbindung zur Gui
     */
    public GameOfLifeMenu(Colorizer colorizer, Logic logic, JFrame mainframe,
            Gui gui) {
        this.colorizer = colorizer;
        this.logic = logic;
        this.gui = gui;

        JMenuBar menuBar = new JMenuBar();
        colorizer.setColors(menuBar);
        mainframe.setJMenuBar(menuBar);

        menuBar.add(createTheFileMenu());
        menuBar.add(createTheExtraMenu());
        menuBar.add(createTheHelpMenu());
    }

    /**
     * Erzeugt das Dateimenü.
     *
     * @return Menüeintrag
     */
    private JMenu createTheFileMenu() {
        JMenu fileMenu = new JMenu("Datei");
        colorizer.setColors(fileMenu);
        fileMenu.setMnemonic('d');

        //fileMenu.addSeparator();

        {
            JMenuItem exitMenuItem = new JMenuItem("Programm beenden", 'b');
            exitMenuItem.setAccelerator(KeyStroke.getKeyStroke(
                    KeyEvent.VK_F4, InputEvent.ALT_DOWN_MASK));
            colorizer.setColors(exitMenuItem);
            fileMenu.add(exitMenuItem);

            exitMenuItem.addActionListener(new ActionListener() {
                @Override
                public void actionPerformed(ActionEvent e) {
                    logic.quit();
                }
            });
        }

        return fileMenu;
    }

    /**
     * Erzeugt das Extramenü.
     *
     * @return Menüeintrag
     */
    private JMenu createTheExtraMenu() {
        JMenu extraMenu = new JMenu("Extra");
        colorizer.setColors(extraMenu);
        extraMenu.setMnemonic('e');

        {
            JMenuItem optionsMenuItem = new JMenuItem(
                    "Optionen", 'o');
            optionsMenuItem.setAccelerator(KeyStroke.getKeyStroke(
                    KeyEvent.VK_F5, 0));
            colorizer.setColors(optionsMenuItem);
            extraMenu.add(optionsMenuItem);

            optionsMenuItem.addActionListener(new ActionListener() {
                @Override
                public void actionPerformed(ActionEvent e) {
                    logic.showOptionDialog();
                }
            });
        }

        return extraMenu;
    }

    /**
     * Erzeugt das Hilfemenü.
     *
     * @return Menüeintrag
     */
    private JMenu createTheHelpMenu() {
        JMenu helpMenu = new JMenu("Hilfe");
        colorizer.setColors(helpMenu);
        helpMenu.setMnemonic('h');

        {
            JMenuItem aboutMenuItem = new JMenuItem("Über das Programm ...",
                    252); // ü ascii
            colorizer.setColors(aboutMenuItem);
            helpMenu.add(aboutMenuItem);
            aboutMenuItem.addActionListener(new ActionListener() {
                @Override
                public void actionPerformed(ActionEvent e) {
                    gui.about();
                }
            });
        }

        {
            JMenuItem helpMenuItem = new JMenuItem(
                    "Hilfe", 'h');
            helpMenuItem.setAccelerator(KeyStroke.getKeyStroke(
                    KeyEvent.VK_F1, 0));
            colorizer.setColors(helpMenuItem);
            helpMenu.add(helpMenuItem);

            helpMenuItem.addActionListener(new ActionListener() {
                @Override
                public void actionPerformed(ActionEvent e) {
                    gui.help();
                }
            });
        }

        {
            JMenuItem todoMenuItem = new JMenuItem(
                    "Weiterentwicklung (todo)", 'w');
            colorizer.setColors(todoMenuItem);
            helpMenu.add(todoMenuItem);

            todoMenuItem.addActionListener(new ActionListener() {
                @Override
                public void actionPerformed(ActionEvent e) {
                    gui.todo();
                }
            });
        }

        {
            JMenuItem changeMenuItem = new JMenuItem(
                    "Neuigkeiten / Änderungen", 'n');
            colorizer.setColors(changeMenuItem);
            helpMenu.add(changeMenuItem);

            changeMenuItem.addActionListener(new ActionListener() {
                @Override
                public void actionPerformed(ActionEvent e) {
                    gui.changes();
                }
            });
        }

        return helpMenu;
    }

}
