package de.duehl.gameoflife.ui.dialogs;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.Image;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JColorChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;

import de.duehl.gameoflife.data.Options;

import de.duehl.swing.ui.colors.Colorizer;
import de.duehl.swing.ui.dialogs.base.ModalDialogBase;
import de.duehl.swing.ui.layout.VerticalLayout;

/**
 * Diese Klasse erzeugt die grafische Oberfläche des Optionendialoges von GameOfLife.
 *
 * @version 1.01     2019-07-08
 * @author Christian Dühl
 */

public class OptionDialog extends ModalDialogBase {

       /** Programmoptionen. */
    private final Options options;

    /** Rahmen der Oberfläche, vor der dieser Dialog erzeugt wird. */
    private final JFrame parentFrame;

    /** Farbverwaltung für die Gui. */
    private final Colorizer colorizer;

    /** Gemerkte Farbe der lebenden Zellen: */
    private Color livingCellColor;

    /** Gemerkte Farbe der toten Zellen: */
    private Color deadCellColor;

    /**
     * Der Konstruktor.
     *
     * @param options
     *            Optionen des Programms.
     * @param parentFrame
     *            Rahmen der Oberfläche, vor der dieser Dialog erzeugt wird.
     * @param colorizer
     *            Farbverwaltung für die Gui.
     * @param programImage
     *            Anzuzeigendes ProgrammIcon.
     */
    public OptionDialog(Options options, JFrame parentFrame,
            Colorizer colorizer, Image programImage) {
        super(parentFrame.getLocation(), programImage, "Optionen", colorizer);
        setColors(getWindowAsComponent());
        setPreferredSize(new Dimension(550, 600));

        this.options = options;
        this.parentFrame = parentFrame;
        this.colorizer = colorizer;

        String livingCellHexColor = options.getLivingCellColor();
        livingCellColor = colorizer.hexColorToJavaColor(livingCellHexColor);
        String deadCellHexColor = options.getDeadCellColor();
        deadCellColor = colorizer.hexColorToJavaColor(deadCellHexColor);

        fillDialog();
    }

    /** Baut die Gui auf. */
    @Override
    protected void populateDialog() {
        add(createCenterPart(), BorderLayout.CENTER);
        add(createLowerPart(), BorderLayout.SOUTH);
    }

    /**
     * Erzeugt den mittleren Bereich mit den Workern:
     *
     * @return Erzeugtes Panel.
     */
    private Component createCenterPart() {
        JPanel panel = new JPanel();
        setColors(panel);

        //setStandardBorder(panel, "Optionen");
        panel.setLayout(new VerticalLayout(3, VerticalLayout.BOTH));

        panel.add(createLivingCellColorPanel());
        panel.add(createDeadCellColorPanel());

        return panel;
    }

    /** Panel für die Farbe der lebendigen Zellen. */
    private Component createLivingCellColorPanel() {
        JPanel panel = new JPanel();
        setColors(panel);
        panel.setLayout(new GridLayout(2, 1, 0, 0));

        JLabel label = new JLabel("Farbe der lebendigen Zellen:");
        setColors(label);
        panel.add(label);

        JPanel subPanel = new JPanel();
        setColors(subPanel);
        panel.add(subPanel);
        subPanel.setLayout(new BorderLayout());

        final JButton button = new JButton("ändern");
        button.setForeground(colorizer.anticolor(livingCellColor));
        button.setBackground(livingCellColor);
        subPanel.add(button, BorderLayout.CENTER);
        button.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent arg0) {
                Color selectedColor = JColorChooser.showDialog(
                        parentFrame,
                        "Eine neue Farbe für die lebendigen Zellen wählen:",
                        livingCellColor
                        );
                if (null != selectedColor) {
                    livingCellColor = selectedColor;
                    button.setForeground(colorizer.anticolor(livingCellColor));
                    button.setBackground(livingCellColor);
                }
            }
        });

        return panel;
    }

    /** Panel für die Farbe der toten Zellen. */
    private Component createDeadCellColorPanel() {
        JPanel panel = new JPanel();
        setColors(panel);
        panel.setLayout(new GridLayout(2, 1, 0, 0));

        JLabel label = new JLabel("Farbe der toten Zellen:");
        setColors(label);
        panel.add(label);

        JPanel subPanel = new JPanel();
        setColors(subPanel);
        panel.add(subPanel);
        subPanel.setLayout(new BorderLayout());

        final JButton button = new JButton("ändern");
        button.setForeground(colorizer.anticolor(deadCellColor));
        button.setBackground(deadCellColor);
        subPanel.add(button, BorderLayout.CENTER);
        button.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent arg0) {
                Color selectedColor = JColorChooser.showDialog(
                        parentFrame,
                        "Eine neue Farbe für die toten Zellen wählen:",
                        deadCellColor
                        );
                if (null != selectedColor) {
                    deadCellColor = selectedColor;
                    button.setForeground(colorizer.anticolor(deadCellColor));
                    button.setBackground(deadCellColor);
                }
            }
        });

        return panel;
    }

    /**
     * Erzeugt den unteren Bereich mit den Buttons.
     *
     * @return Erzeugtes Panel.
     */
    private Component createLowerPart() {
        JPanel panel = new JPanel();
        setColors(panel);
        panel.setLayout(new BorderLayout());

        JButton quitButton = new JButton("Abbrechen");
        setColors(quitButton);
        panel.add(quitButton, BorderLayout.WEST);
        quitButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent arg0) {
                closeDialog();
            }
        });

        JButton okButton = new JButton("Änderungen übernehmen");
        setColors(okButton);
        panel.add(okButton, BorderLayout.EAST);
        okButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent arg0) {
                apply();
            }
        });

        return panel;
    }

    /**
     * Wertet nach dem Klick auf Änderungen übernehmen die gewählten
     * Einstellungen aus und schreibt diese in die Optionen.
     */
    private void apply() {
        String livingHex = colorizer.javaColorToHexColor(livingCellColor);
        options.setLivingCellColor(livingHex);

        String deadHex = colorizer.javaColorToHexColor(deadCellColor);
        options.setDeadCellColor(deadHex);

        closeDialog();
    }

}
