package de.duehl.gameoflife.ui.panel;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JPanel;
import javax.swing.SwingUtilities;

import de.duehl.swing.ui.PaintableAreaCalculator;
import de.duehl.swing.ui.colors.ColorTool;
import de.duehl.gameoflife.data.Options;
import de.duehl.gameoflife.logic.animationloop.AnimationStepWatcher;
import de.duehl.gameoflife.logic.field.GameField;
import de.duehl.gameoflife.ui.MouseWatcher;

/**
 * Diese Klasse stellt das Spielfeld dar.
 *
 * @version 1.01     2015-11-03
 * @author Christian Dühl
 */

public class GameOfLifePanel extends JPanel implements MouseWatcher,
        AnimationStepWatcher {

    private static final long serialVersionUID = 3930701984112580372L;

    /** Farbverwaltung für die Gui. */
    private final ColorTool colorTool;

    /** Spielfeld. */
    private volatile GameField field;

    /** Gib an, ob der Edit-Mode aktiv ist. */
    private boolean editMode;

    /** Menge der dargestellten Zellen. */
    private final Cells cells;

    /** Menge der im Edit-Mode mit der Maus berührten Zellen bis zu loslassen. */
    private final List<Cell> touchedCelles;

    /** Gibt beim Editieren an, ob mit der Maus selektierte Zellen leben sollen oder nicht. */
    private boolean editCellToLiving;

    /** Programmoptionen. */
    private final Options options;

    /**
     * Konstruktor.
     *
     * @param field
     *            Spielfeld.
     * @param options
     *            Programmoptionen
     */
    public GameOfLifePanel(final GameField field,
            final Options options) {
        this.field = field;
        this.options = options;

        colorTool = new ColorTool();

        MouseWatcher watcher = (MouseWatcher) this;
        addMouseListener(new GameOfLifeMouseListener(watcher));
        addMouseMotionListener(new GameOfLifeMouseMotionListener(watcher));

        cells = new Cells();
        touchedCelles = new ArrayList<>();
    }

    /**
     * Auf Panel zeichnen.
     *
     * @param graphics
     *            Grafikobjekt.
     */
    @Override
    public void paintComponent(Graphics graphics) {
        super.paintComponent(graphics);

        paintField(graphics);
    }

    /**
     * Zeichnet das Spielfeld.
     *
     * @param graphics
     *            Grafikobjekt.
     */
    private void paintField(Graphics graphics) {
        /* Größe des Spielfeldes bestimmen: */
        final int fieldWidth = field.getWidth();
        final int fieldHeight = field.getHeight();

        /* Die wirklich Zeichenfläche ermitteln: */
        PaintableAreaCalculator paint = new PaintableAreaCalculator(this);

        /* Ausmaße der wirklichen Zeichenfläche: */
        final int panelWidth = paint.getPaintableWidth();
        final int panelHeight = paint.getPaintableHeight();

        /* Obere, linke Ecke: */
        final int upperLeftX = paint.getUpperLeftX();
        final int upperLeftY = paint.getUpperLeftY();

        /* Untere, rechte Ecke: */
        final int lowerRightX = paint.getLowerRightX();
        final int lowerRightY = paint.getLowerRightY();

        /* Rechteck der Zeichenfläche darstellen, falls gewünscht: */
        boolean paintBorder = false;
        if (paintBorder) {
            graphics.setColor(Color.RED);
            graphics.drawLine(upperLeftX, upperLeftY, lowerRightX, upperLeftY);
            graphics.drawLine(lowerRightX, upperLeftY, lowerRightX, lowerRightY);
            graphics.drawLine(lowerRightX, lowerRightY, upperLeftX, lowerRightY);
            graphics.drawLine(upperLeftX, lowerRightY, upperLeftX, upperLeftY);

            graphics.setColor(Color.GREEN);
            graphics.drawRect(upperLeftX, upperLeftY, panelWidth, panelHeight);
        }

        /* Die Anzahl Pixel pro Zeile oder Spalte bestimmen: */
        double widthRatio = (1.0 * panelWidth) / fieldWidth;
        double heightRatio = (1.0 * panelHeight) / fieldHeight;

        /*
         * Weil die Einzelnen Zellen quadratisch sein sollen, nehmen wir das
         * kleinere Verhältnis:
         */
        double ratio = Math.min(widthRatio, heightRatio);
        int pixel = (int) ratio;

        /* Gitterbreite bestimmen: */
        int gridPixel = determineGridWidth(pixel);

        /*
         * Da der Rand rechts und unten auch dargestellt werden soll, muss er
         * mit hineinpassen. Also wird solange Pixel (und gridPixel)
         * verkleinert, bis es ganz auf das Panel passt:
         */
        boolean fits = false;
        int neededX = -1;
        int neededY = -1;
        while (!fits) {
            neededX = calculateNeeded(fieldWidth, pixel, gridPixel);
            neededY = calculateNeeded(fieldHeight, pixel, gridPixel);
            if (neededX <= panelWidth && neededY <= panelHeight) {
                fits = true;
            }
            else {
                --pixel;
                gridPixel = determineGridWidth(pixel);
            }
        }

        /* Anzahl der nicht verwendeten Pixel bestimmen: */
        int emptyX = panelWidth - neededX;
        int emptyY = panelHeight - neededY;

        /* Startpunkt für die Darstellung bestimmen: */
        int startX = upperLeftX + emptyX / 2;
        int startY = upperLeftY + emptyY / 2;

        /* Echte Breite und Höhe der Zellen: */
        int pixelWithoutGrid = pixel - gridPixel;

        /* Farben für die lebenden und toten Zellen festlegen: */
        String aliveHexColor = options.getLivingCellColor();
        Color aliveColor = colorTool.hexColorToJavaColor(aliveHexColor);
        String deadHexColor = options.getDeadCellColor();
        Color deadColor = colorTool.hexColorToJavaColor(deadHexColor);

        /* Zellen zeichnen: */
        synchronized (cells) {
            cells.clear();
            for (int y=0; y<fieldHeight; ++y) {
                int pixelY = startY + y * pixel;
                for (int x=0; x<fieldWidth; ++x) {
                    int pixelX = startX + x * pixel;

                    Color color;
                    if (field.isCellAlive(x, y)) {
                        color = aliveColor;
                    }
                    else {
                        color = deadColor;
                    }
                    graphics.setColor(color);

                    graphics.fillRect(pixelX, pixelY, pixelWithoutGrid,
                            pixelWithoutGrid);

                    Cell cell = new Cell(x, y,
                            new Rectangle(pixelX, pixelY, pixelWithoutGrid,
                                    pixelWithoutGrid));
                    cells.add(cell);
                }
            }
        }
    }

    /**
     * Berechnet aus der Breite oder Höhe des Spielfeldes die benötigte Anzahl
     * für die Darstellung aller Zellen und des Gitters.
     *
     * @param fieldWidthOrHeight
     *            Breite oder Höhe des Spielfeldes
     * @param pixel
     *            Anzahl Pixel pro Zelle in x- und y-Richtung.
     * @param gridPixel
     *            Breite des Gitters
     * @return Benötigte Anzahl an Pixeln für die Darstellung aller Zellen und
     *         des Gitters in dieser Richtung.
     */
    private int calculateNeeded(final int fieldWidthOrHeight, final int pixel,
            final int gridPixel) {
        /*
         * Das Gitter außen wird nicht dargestellt. Von jeder Zellenbreite oder
         * Höhe (pixel) geht ein Gitter ab.
         *
         * Von der letzten Zelle muss kein Gitter mehr abgehen, da die
         * Außenumrandung des Gitters nicht dargestellt wird. Wollte man sie
         * darstellen, so müsste man eine Gitterbreite addieren, statt sie
         * subtrahieren.
         *
         * Dann müsste man aber auch bei startX und startY oben die
         * Gitterbreite noch draufschlagen.
         */
        return fieldWidthOrHeight * pixel - gridPixel;
    }

    /**
     * Berechnet aus der Anzahl von Pixeln für eine Zelle die breite des
     * Gitters.
     *
     * @param pixel
     *            Anzahl Pixel pro Zelle in x- und y-Richtung.
     * @return Breite des Gitters.
     */
    private int determineGridWidth(final int pixel) {
        /* Pro dreizehn Spielfeldpixeln ein Gitterpixel: */
        int gridWidth = pixel / 13;

        /*
         * Verschwinden soll das Gitter aber nicht, es sei denn, das Gitter
         * würde dann das Spielfeld schlucken:
         */
        if (gridWidth < 1 && pixel > 1) {
            gridWidth = 1;
        }

        return gridWidth;
    }

    /** Setter für den Edit-Mode. */
    public void setEditMode(boolean editMode) {
        this.editMode = editMode;
    }

    /**
     * Meldet einen Mausklick.
     *
     * @param mouseX
     *            x-Koordinate des Mausklicks.
     * @param mouseY
     *            y-Koordinate des Mausklicks.
     * @param buttonNumber
     *            Nummer des Mausknopfes (vgl. java.awt.event.MouseEvent)
     *            1:links, 2:mitte, 3:rechts
     *            Zur Abfrage kann MouseEvent.BUTTON1 etc. verwendet werden.
     */
    @Override
    public void mousePressedAt(final int mouseX, final int mouseY,
            final int buttonNumber) {
        if (editMode) {
            touchedCelles.clear();
            if (MouseEvent.BUTTON1 == buttonNumber) {
                editCellToLiving = true;
            }
            else if (MouseEvent.BUTTON3 == buttonNumber) {
                editCellToLiving = false;
            }
            else {
                System.out.println("UNBEKANNTE BUTTONNUMMER!!! " + buttonNumber);
            }

            determineAndAddCell(mouseX, mouseY);
        }
    }

    /**
     * Meldet das Ziehen der Maus während eines Mausklicks.
     *
     * @param mouseX
     *            x-Koordinate des Mausklicks.
     * @param mouseY
     *            y-Koordinate des Mausklicks.
     * @param buttonNumber
     *            Nummer des Mausknopfes (vgl. java.awt.event.MouseEvent)
     *            1:links, 2:mitte, 3:rechts
     *            Zur Abfrage kann MouseEvent.BUTTON1 etc. verwendet werden.
     */
    @Override
    public void mouseDraggedTo(final int mouseX, final int mouseY,
            final int buttonNumber) {
        if (editMode) {
            determineAndAddCell(mouseX, mouseY);
        }
    }

    /**
     * Meldet das Lösen des Mausklicks.
     *
     * @param mouseX
     *            x-Koordinate des Mausklicks.
     * @param mouseY
     *            y-Koordinate des Mausklicks.
     * @param buttonNumber
     *            Nummer des Mausknopfes (vgl. java.awt.event.MouseEvent)
     *            1:links, 2:mitte, 3:rechts
     *            Zur Abfrage kann MouseEvent.BUTTON1 etc. verwendet werden.
     */
    @Override
    public void mouseReleasedAt(final int mouseX, final int mouseY,
            final int buttonNumber) {
        if (editMode) {
            determineAndAddCell(mouseX, mouseY);
        }
    }

    /**
     * Meldet das Lösen des Mausklicks.
     *
     * @param mouseX
     *            x-Koordinate des Mausklicks.
     * @param mouseY
     *            y-Koordinate des Mausklicks.
     */
    private void determineAndAddCell(final int mouseX, final int mouseY) {
        synchronized (cells) {
            Cell cell = cells.getCellStruckedby(new Point(mouseX, mouseY));
            if (null != cell && !touchedCelles.contains(cell)) {
                touchedCelles.add(cell);
                field.setCellLiving(cell.getX(), cell.getY(), editCellToLiving);
            }
        }
        repaint();
    }

    /** Berechnet die nächste Generation und zeigt sie an. */
    public void nextGeneration() {
        field.calculateNextGeneration();
        repaint();
    }

    /**
     * Wird aufgerufen, wenn in Animation ein neuer Schritt ausgerechnet wurde.
     */
    @Override
    public void nextGenerationCalculated() {
        SwingUtilities.invokeLater(new Runnable() {
            @Override
            public void run() {
                repaint();
            }
        });
    }

}
