package de.duehl.basics.io;

/*
 * Copyright 2016 Christian Dühl. All rights reserved.
 *
 * This program is free software. You can redistribute it and/or
 * modify it under the same terms as perl:
 *
 * general:  http://dev.perl.org/licenses/
 * GPL:      http://dev.perl.org/licenses/gpl1.html
 * artistic: http://dev.perl.org/licenses/artistic.html
 */

/**
 * Diese Klasse bietet Methoden rund um die Behandlung von Texten.
 *
 * @version 1.01     2016-10-14
 * @author Christian Dühl
 */

public class TextFileCompare {

    /** Zeichensatz der einzulesenden Datei (z.B. ISO_8859_1, UTF_8). */
    private final Charset charset;

    /** Konstruktor. */
    public TextFileCompare() {
        this(Charset.ISO_8859_1);
    }

    /**
     * Konstruktor.
     *
     * @param charset
     *            Zeichensatz der einzulesenden Datei (z.B. ISO_8859_1).
     */
    public TextFileCompare(Charset charset) {
        this.charset = charset;
    }

    /**
     * Prüft die beiden übergebenen Dateien auf inhaltliche Gleichheit.
     *
     * @param firstFilename
     *            Name der ersten zu vergleichenden Datei.
     * @param secondFilename
     *            Name der zweiten zu vergleichenden Datei.
     * @return Wahrheitswert, true genau dann, wenn beide Dateien existieren und inhaltlich
     *         identisch sind.
     */
    public boolean isContentEqual(String firstFilename, String secondFilename) {
        if (!FileHelper.exists(firstFilename)) {
            return false;
        }
        if (!FileHelper.exists(secondFilename)) {
            return false;
        }

        Reader firstReader = new FineFileReader(firstFilename, charset);
        Reader secondReader = new FineFileReader(secondFilename, charset);

        boolean loop = true;
        boolean equals = true;
        while (loop) {
            String firstLine = firstReader.readNextLine();
            String secondLine = secondReader.readNextLine();

            /* Beide Dateien sind komplett eingelesen: */
            if (null == firstLine && null == secondLine) {
                loop = false;
            }
            /* Nur die erste Datei ist komplett eingelesen: */
            else if (null == firstLine) {
                loop = false;
                equals = false;
            }
            /* Nur die zweite Datei ist komplett eingelesen: */
            else if (null == secondLine) {
                loop = false;
                equals = false;
            }
            /* Anderenfalls inhaltlicher Vergleich der beiden Zeilen: */
            else if  (!firstLine.equals(secondLine)) {
                loop = false;
                equals = false;
            }
        }

        secondReader.close();
        firstReader.close();

        return equals;
    }

}
