package de.duehl.swing.ui.colors;

/*
 * Copyright 2016 Christian Dühl. All rights reserved.
 *
 * This program is free software. You can redistribute it and/or
 * modify it under the same terms as perl:
 *
 * general:  http://dev.perl.org/licenses/
 * GPL:      http://dev.perl.org/licenses/gpl1.html
 * artistic: http://dev.perl.org/licenses/artistic.html
 */

import java.awt.Color;
import java.util.ArrayList;
import java.util.List;

/**
 * Diese Klasse stellt eine Liste von benannten Farbe dar und bietet
 * verschiedene Methoden, benannte Farben oder Swing-Farben aus dieser Liste
 * zu erhalten.
 *
 * @version 1.01     2014-07-31
 * @author Christian Dühl
 */

public class NamedColorList {

    /** Liste mit den benannten Farben. */
    private final List<NamedColor> namedColors;

    /** Konstruktor. */
    public NamedColorList() {
        namedColors = new ArrayList<>();
    }

    /** Fügt eine Farbe zur Liste hinzu. */
    public void add(NamedColor namedColor) {
        if (null != getNamedColorByName(namedColor.getName())) {
            throw new IllegalArgumentException("Eine Farbe mit dem Namen '"
                    + namedColor.getName() + "' ist bereits vorhanden!");
        }
        namedColors.add(namedColor);
    }

    /**
     * Getter für eine benannte Farbe über ihren Namen.
     *
     * @param name
     *            Name der Farbe.
     * @return Benannte Farbe oder null, falls keine Farbe unter diesem Namen
     *         bekannt war.
     */
    public NamedColor getNamedColorByName(String name) {
        for (NamedColor namedColor : namedColors) {
            if (namedColor.getName().equals(name)) {
                return namedColor;
            }
        }
        return null;
    }

    /**
     * Getter für eine benannte Farbe über ihre hexadezimale Stringdarstellung.
     *
     * @param hexValue
     *            Hexadezimale Stringdarstellung der Farbe.
     * @return Benannte Farbe oder null, falls keine Farbe mit dieser
     *         hexadezimalen Stringdarstellung bekannt war.
     */
    public NamedColor getNamedColorByHexcode(String hexValue) {
        for (NamedColor namedColor : namedColors) {
            if (namedColor.getHexValue().equals(hexValue)) {
                return namedColor;
            }
        }
        return null;
    }

    /**
     * Getter für eine Swing-Farbe über ihren Namen.
     *
     * @param name
     *            Name der Farbe.
     * @return Swing-Farbe oder null, falls keine Farbe unter diesem Namen
     *         bekannt war.
     */
    public Color getColorByName(String name) {
        NamedColor namedColor = getNamedColorByName(name);
        if (null == namedColor) {
            return null;
        }
        else {
            return namedColor.getColor();
        }
    }

    /**
     * Getter für eine Swing-Farbe über ihre hexadezimale Stringdarstellung.
     *
     * @param hexValue
     *            Hexadezimale Stringdarstellung der Farbe.
     * @return Swing-Farbe oder null, falls keine Farbe mit dieser
     *         hexadezimalen Stringdarstellung bekannt war.
     */
    public Color getColorByHexcode(String hexValue) {
        NamedColor namedColor = getNamedColorByHexcode(hexValue);
        if (null == namedColor) {
            return null;
        }
        else {
            return namedColor.getColor();
        }
    }

    /**
     * Getter für die hexadezimale Stringdarstellung einer Farbe über ihren
     * Namen.
     *
     * @param name
     *            Name der Farbe.
     * @return Hexadezimale Stringdarstellung oder null, falls keine Farbe unter
     *         diesem Namen bekannt war.
     */
    public String getHexCodeFromNamedColorByName(String name) {
        NamedColor namedColor = getNamedColorByName(name);
        if (null == namedColor) {
            return null;
        }
        else {
            return namedColor.getHexValue();
        }
    }

}
