package de.duehl.swing.ui.components.selections;

/*
 * Copyright 2018 Christian Dühl. All rights reserved.
 *
 * This program is free software. You can redistribute it and/or
 * modify it under the same terms as perl:
 *
 * general:  http://dev.perl.org/licenses/
 * GPL:      http://dev.perl.org/licenses/gpl1.html
 * artistic: http://dev.perl.org/licenses/artistic.html
 */

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.FocusListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import de.duehl.swing.ui.GuiTools;
import de.duehl.swing.ui.colors.Colorizer;

/**
 * Diese Klasse stellt die Auswahl eines Textes etwa in einem Optionendialog dar.
 *
 * @version 1.01     2018-04-05
 * @author Christian Dühl
 */

public class StringSelection {

    /** Überschrift für die Wahl dieser Datei oder dieses Verzeichnisses. */
    private final JLabel titleLabel;

    /** Panel mit Überschrift und Eingabefeld. */
    private final JPanel mainPanel;

    /** Textfeld mit der einzugebenden Option. */
    private final JTextField textField;

    private final Component textFieldPanel;

    /**
     * Konstruktor.
     *
     * @param title
     *            Überschrift für die Wahl dieser Datei oder dieses Verzeichnisses.
     */
    public StringSelection(String title) {
        titleLabel = new JLabel(title);
        textField = new JTextField();
        GuiTools.setEditFieldColors(textField);
        textFieldPanel = createTextField();
        mainPanel = createPanel();
    }

    /** Erstellt das Eingabefeld. */
    private Component createTextField() {
        JPanel panel = new JPanel();
        panel.setLayout(new BorderLayout());

        panel.add(textField, BorderLayout.CENTER);

        return panel;
    }

    /** Erstellt den Panel mit der Überschrift und dem Eingabefeld in vertikaler Anordnung. */
    private JPanel createPanel() {
        JPanel panel = new JPanel();
        panel.setLayout(new BorderLayout());

        panel.add(titleLabel, BorderLayout.NORTH);
        panel.add(textFieldPanel, BorderLayout.CENTER);

        return panel;
    }

    /** Getter für die (getrimmte) Option. */
    public String getText() {
        String text = textField.getText();
        text = text.trim();
        return text;
    }

    /** Setter für die Option. */
    public void setText(String text) {
        textField.setText(text);
    }

    /** Getter für das Panel mit Überschrift und Eingabefeld. */
    public Component getPanel() {
        return mainPanel;
    }

    /** Färbt alle Komponenten mit dem übergebenen Colorizer ein. */
    public void colorize(Colorizer colorizer) {
        colorizer.setColors(mainPanel);
        colorizer.setColors(titleLabel);
        colorizer.setColors(textField);
        colorizer.setEditFieldColors(textField);
    }

    /** Setter für die gewünschte Größe des Textfeldes. */
    public void setPreferredSize(Dimension preferredSize) {
        textField.setPreferredSize(preferredSize);
    }

    /** Fügt einen FocusListener zum Textfeld hinzu. */
    public void addTextFieldFocuslistener(FocusListener focusListener) {
        textField.addFocusListener(focusListener);
    }

    /** Legt fest, ob das Textfeld editierbar ist. */
    public void setEditable(boolean editable) {
        textField.setEditable(editable);
    }

    /** Setzt den Focus in das Feld. */
    public void requestFocus() {
        textField.requestFocus();
    }

    /** Ändert die Anordnung von Überschrift und Textfeld in horizontal, Überschrift links. */
    public void makeHorizontal() {
        mainPanel.removeAll();
        mainPanel.add(titleLabel, BorderLayout.WEST);
        mainPanel.add(textFieldPanel, BorderLayout.CENTER);
    }

    /** Fügt einen KeyListener zum Textfeld hinzu. */
    public void addKeyListener(KeyListener keyListener) {
        textField.addKeyListener(keyListener);
    }

    /** Fügt einen KeyListener zum Textfeld hinzu, der bei Return die übergebene Aktion ausführt. */
    public void addReturnListener(Runnable runnable) {
        textField.addKeyListener(new KeyListener() {
            @Override
            public void keyTyped(KeyEvent event) {
            }
            @Override
            public void keyReleased(KeyEvent event) {
            }
            @Override
            public void keyPressed(KeyEvent event) {
                if (event.getKeyCode() == KeyEvent.VK_ENTER && event.getModifiersEx() == 0) {
                    runnable.run();
                }
            }
        });
    }

}
