package de.duehl.swing.ui.dialogs.base;

/*
 * Copyright 2019 Christian Dühl. All rights reserved.
 *
 * This program is free software. You can redistribute it and/or
 * modify it under the same terms as perl:
 *
 * general:  http://dev.perl.org/licenses/
 * GPL:      http://dev.perl.org/licenses/gpl1.html
 * artistic: http://dev.perl.org/licenses/artistic.html
 */

import java.awt.Dimension;
import java.awt.Image;
import java.awt.Point;
import java.awt.Dialog.ModalityType;

import javax.swing.JDialog;
import javax.swing.JMenuBar;
import javax.swing.JRootPane;

import de.duehl.swing.logic.Quitter;
import de.duehl.swing.ui.colors.Colorizer;
import de.duehl.swing.ui.dialogs.DialogHelper;
import de.duehl.swing.ui.listener.ClosingWindowListener;

/**
 * Diese Klasse ist die abstrakte Basis der Dialoge mit modalem JDialog.
 *
 * @version 1.01     2019-05-24
 * @author Christian Dühl
 */

public abstract class ModalDialogBase extends AbstractDialogBase  {

    /** Der Dialog, der dargestellt wird. */
    private final JDialog dialog;

    /** Gibt an, ob ein ClosingWindowListener benutzt wird. */
    private boolean weUseAClosingWindowListener;


    /**
     * Konstruktor für ein gepacktes Fenster ohne fest vorgegebene Dimension.
     *
     * @param dialogTitle
     *            Titel des Dialoges.
     */
    public ModalDialogBase(String dialogTitle) {
        this(new Point(0, 0), null, dialogTitle, IGNORE_DIMENSION, null);
        ignoreSize();
    }

    /**
     * Konstruktor.
     *
     * @param dialogTitle
     *            Titel des Dialoges.
     * @param dialogDimension
     *            Größe des Dialogs.
     */
    public ModalDialogBase(String dialogTitle, Dimension dialogDimension) {
        this(new Point(0, 0), null, dialogTitle, dialogDimension, null);
    }

    /**
     * Konstruktor für ein gepacktes Fenster ohne fest vorgegebene Dimension.
     *
     * @param parentLocation
     *            Position des Rahmens der Oberfläche, vor der dieser Dialog erzeugt wird.
     * @param programImage
     *            Icon für das Programm.
     * @param dialogTitle
     *            Titel des Dialoges.
     */
    public ModalDialogBase(Point parentLocation, Image programImage, String dialogTitle) {
        this(parentLocation, programImage, dialogTitle, IGNORE_DIMENSION, null);
        ignoreSize();
    }

    /**
     * Konstruktor.
     *
     * @param parentLocation
     *            Position des Rahmens der Oberfläche, vor der dieser Dialog erzeugt wird.
     * @param programImage
     *            Icon für das Programm.
     * @param dialogTitle
     *            Titel des Dialoges.
     * @param dialogDimension
     *            Größe des Dialogs.
     */
    public ModalDialogBase(Point parentLocation, Image programImage, String dialogTitle,
            Dimension dialogDimension) {
        this(parentLocation, programImage, dialogTitle, dialogDimension, null);
    }

    /**
     * Konstruktor für ein gepacktes Fenster ohne fest vorgegebene Dimension.
     *
     * @param parentLocation
     *            Position des Rahmens der Oberfläche, vor der dieser Dialog erzeugt wird.
     * @param programImage
     *            Icon für das Programm.
     * @param dialogTitle
     *            Titel des Dialoges.
     * @param colorizer
     *            Farbverwaltung für die Gui.
     */
    public ModalDialogBase(Point parentLocation, Image programImage, String dialogTitle,
            Colorizer colorizer) {
        this(parentLocation, programImage, dialogTitle, IGNORE_DIMENSION, colorizer);
        ignoreSize();
    }

    /**
     * Konstruktor.
     *
     * @param parentLocation
     *            Position des Rahmens der Oberfläche, vor der dieser Dialog erzeugt wird.
     * @param programImage
     *            Icon für das Programm.
     * @param dialogTitle
     *            Titel des Dialoges.
     * @param dialogDimension
     *            Größe des Dialogs.
     * @param colorizer
     *            Farbverwaltung für die Gui.
     */
    public ModalDialogBase(Point parentLocation, Image programImage, String dialogTitle,
            Dimension dialogDimension, Colorizer colorizer) {
        super(parentLocation, programImage, dialogTitle, dialogDimension, colorizer);

        dialog = new JDialog();
        setWindow(dialog);
        weUseAClosingWindowListener = false;
    }

    /** Baut die Gui auf. */
    @Override
    protected final void fillDialog() {
        dialog.setTitle(windowTitle);
        dialog.setModalityType(ModalityType.APPLICATION_MODAL);
        if (!weUseAClosingWindowListener) {
            dialog.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
        }
        super.fillDialog();
    }

    /** Setzt den Titel des Rahmens neu. */
    protected final void changeTitle(String newTitle) {
        dialog.setTitle(newTitle);
        refresh();
    }

    /** Getter für den Titel des Dialogs. */
    public String getTitle() {
        return dialog.getTitle();
    }

    /** Setter für den Titel des Dialogs. */
    public void setTitle(String title) {
        changeTitle(title);
    }

    /**
     * Fügt dem Dialog die Tastenkombination Escape hinzu, woraufhin die übergebene Komponente
     * beendet wird.
     */
    protected final void addEscapeBehaviour() {
        DialogHelper.addEscapeBehaviourForJDialog(dialog);
    }

    /** Gibt die RootPane des Dialogs zurück. */
    @Override
    protected final JRootPane getRootPane() {
        return dialog.getRootPane();
    }

    /** Fügt einen Listener zum Beenden des Dialogs über das [x] oben rechts hinzu. */
    public void addClosingWindowListener(Quitter quitter) {
        dialog.setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);
        dialog.addWindowListener(new ClosingWindowListener(quitter));
        weUseAClosingWindowListener = true;
    }

    public JDialog getRealDialog() {
        return dialog;
    }

    /** Trägt das Menü ein. */
    public void addMenu(JMenuBar menuBar) {
        dialog.setJMenuBar(menuBar);
        dialog.validate();
    }

    public void setLocationRelativeToNothing() {
        dialog.setLocationRelativeTo(null);
    }

}
